import os
from typing import Dict, Any, Optional
import yaml
from pydantic_settings import BaseSettings
from pydantic import Field, validator
from pathlib import Path
from enum import Enum


class Environment(str, Enum):
    DEVELOPMENT = "development"
    TESTING = "testing"
    PRODUCTION = "production"


class AppConfig(BaseSettings):
    name: str = Field(..., env="APP_NAME")
    version: str = Field(..., env="APP_VERSION")
    debug: bool = Field(False, env="APP_DEBUG")
    secret_key: str = Field(..., env="APP_SECRET_KEY")
    timezone: str = Field("Asia/Shanghai", env="APP_TIMEZONE")
    locale: str = Field("zh_CN", env="APP_LOCALE")
    api_prefix: str = Field("/api/v1", env="API_PREFIX")
    environment: Environment = Field(Environment.DEVELOPMENT, env="APP_ENV")
    log_level: str = Field("INFO", env="LOG_LEVEL")
    cors_origins: str = Field("*", env="CORS_ORIGINS")
    rate_limit: int = Field(1000, env="RATE_LIMIT")

    @validator("environment", pre=True)
    def validate_environment(cls, v):
        return Environment(v.lower())

    class Config:
        env_file = ".env"
        env_file_encoding = "utf-8"


class DatabaseConfig(BaseSettings):
    host: str = Field(..., env="DB_HOST")
    port: int = Field(3306, env="DB_PORT")
    user: str = Field(..., env="DB_USER")
    password: str = Field(..., env="DB_PASSWORD")
    name: str = Field(..., env="DB_NAME")
    pool_size: int = Field(10, env="DB_POOL_SIZE")
    max_overflow: int = Field(20, env="DB_MAX_OVERFLOW")
    echo: bool = Field(False, env="DB_ECHO")
    pool_recycle: int = Field(3600, env="DB_POOL_RECYCLE")
    ssl_mode: Optional[str] = Field(None, env="DB_SSL_MODE")


class RedisConfig(BaseSettings):
    host: str = Field("localhost", env="REDIS_HOST")
    port: int = Field(6379, env="REDIS_PORT")
    password: Optional[str] = Field(None, env="REDIS_PASSWORD")
    db: int = Field(0, env="REDIS_DB")
    cache_ttl: int = Field(3600, env="CACHE_TTL")


class AuthConfig(BaseSettings):
    token_expire_minutes: int = Field(1440, env="TOKEN_EXPIRE_MINUTES")
    refresh_token_expire_days: int = Field(7, env="REFRESH_TOKEN_EXPIRE_DAYS")
    algorithm: str = Field("HS256", env="TOKEN_ALGORITHM")
    issuer: str = Field("smart-question-bank", env="TOKEN_ISSUER")


class Config(BaseSettings):
    app: AppConfig
    database: DatabaseConfig
    redis: RedisConfig
    auth: AuthConfig

    class Config:
        env_nested_delimiter = "__"


class ConfigLoader:

    def __init__(self):
        self._config_data = {}
        self.config = None
        self._load_config()
        self.config = Config(
            app=AppConfig(**self._get_config_section('app')),
            database=DatabaseConfig(**self._get_config_section('database')),
            redis=RedisConfig(**self._get_config_section('redis')),
            auth=AuthConfig(**self._get_config_section('auth')))

    def _get_config_section(self, section: str) -> dict:
        """获取配置文件中指定section的内容"""
        return self._config_data.get(section, {})

    def _load_config(self):
        # 确定当前环境
        env = os.getenv("APP_ENV", "development")

        # 按顺序加载配置
        self._load_config_file("application.yml")  # 基础配置
        self._load_config_file(f"application-{env}.yml")  # 环境特定配置

    def _load_config_file(self, filename: str):
        """加载指定配置文件"""
        config_path = Path(__file__).parent / filename
        if config_path.exists():
            with open(config_path, "r", encoding="utf-8") as f:
                config = yaml.safe_load(f)
                if config:
                    self._merge_configs(self._config_data, config)

    def _merge_configs(self, base: Dict, update: Dict):
        """递归合并配置"""
        for key, value in update.items():
            if key in base and isinstance(base[key], dict) and isinstance(
                    value, dict):
                self._merge_configs(base[key], value)
            else:
                base[key] = value

    def _get_config_section(self, section: str) -> dict:
        """获取配置文件中指定section的内容"""
        return self._config_data.get(section, {})

    def get_config(self) -> Config:
        return self.config


config_loader = ConfigLoader()
config = config_loader.get_config()
