from flask import request, g
from ..services.auth_service import AuthService
from ..exception.api_exception import ApiException
from ..common.constants import ErrorCode


class AuthMiddleware:
    """认证中间件"""

    @staticmethod
    def auth_required(f):
        """认证装饰器"""

        def wrapper(*args, **kwargs):
            token = request.headers.get('Authorization')
            if not token:
                raise ApiException(ErrorCode.UNAUTHORIZED)

            # 验证token并获取当前用户
            db = g.db
            user = AuthService.get_current_user(db, token)
            if not user:
                raise ApiException(ErrorCode.UNAUTHORIZED)

            # 将当前用户存入g对象
            g.current_user = user
            return f(*args, **kwargs)

        return wrapper

    @staticmethod
    def permission_required(permission_code: str):
        """权限装饰器"""

        def decorator(f):

            def wrapper(*args, **kwargs):
                # 确保用户已认证
                if not hasattr(g, 'current_user'):
                    raise ApiException(ErrorCode.UNAUTHORIZED)

                # 获取用户所有权限
                user_permissions = set()
                for role in g.current_user.roles:
                    for permission in role.permissions:
                        user_permissions.add(permission.code)

                # 检查权限
                if permission_code not in user_permissions:
                    raise ApiException(ErrorCode.FORBIDDEN)

                return f(*args, **kwargs)

            return wrapper

        return decorator

    @staticmethod
    def role_or_permission_required(role_name: str = None,
                                    permission_code: str = None):
        """角色或权限装饰器"""

        def decorator(f):

            def wrapper(*args, **kwargs):
                # 确保用户已认证
                if not hasattr(g, 'current_user'):
                    raise ApiException(ErrorCode.UNAUTHORIZED)

                # 检查角色
                if role_name:
                    user_roles = [role.name for role in g.current_user.roles]
                    if role_name in user_roles:
                        return f(*args, **kwargs)

                # 检查权限
                if permission_code:
                    user_permissions = set()
                    for role in g.current_user.roles:
                        for permission in role.permissions:
                            user_permissions.add(permission.code)

                    if permission_code in user_permissions:
                        return f(*args, **kwargs)

                raise ApiException(ErrorCode.FORBIDDEN)

            return wrapper

        return decorator
