from datetime import datetime
from typing import Any, Dict, Optional
from sqlalchemy import Column, Integer, DateTime
from sqlalchemy.ext.declarative import as_declarative, declared_attr
from sqlalchemy.orm import Session
from sqlalchemy.sql import func


@as_declarative()
class BaseModel:
    id = Column(Integer, primary_key=True, index=True, autoincrement=True)
    created_at = Column(DateTime, server_default=func.now(), comment="创建时间")
    updated_at = Column(DateTime,
                        server_default=func.now(),
                        onupdate=func.now(),
                        comment="更新时间")
    deleted_at = Column(DateTime, nullable=True, comment="删除时间")

    @declared_attr
    def __tablename__(cls) -> str:
        return cls.__name__.lower()

    def to_dict(self) -> Dict[str, Any]:
        """将模型实例转换为字典"""
        return {c.name: getattr(self, c.name) for c in self.__table__.columns}

    @classmethod
    def create(cls, db: Session, **kwargs) -> "BaseModel":
        """创建新记录"""
        instance = cls(**kwargs)
        db.add(instance)
        db.commit()
        db.refresh(instance)
        return instance

    @classmethod
    def get(cls, db: Session, id: int) -> Optional["BaseModel"]:
        """根据ID获取记录"""
        return db.query(cls).filter(cls.id == id).first()

    @classmethod
    def update(cls, db: Session, id: int, **kwargs) -> Optional["BaseModel"]:
        """更新记录"""
        instance = cls.get(db, id)
        if instance:
            for key, value in kwargs.items():
                setattr(instance, key, value)
            db.commit()
            db.refresh(instance)
        return instance

    @classmethod
    def delete(cls, db: Session, id: int) -> bool:
        """软删除记录"""
        instance = cls.get(db, id)
        if instance:
            instance.deleted_at = datetime.now()
            db.commit()
            return True
        return False

    @classmethod
    def hard_delete(cls, db: Session, id: int) -> bool:
        """硬删除记录"""
        instance = cls.get(db, id)
        if instance:
            db.delete(instance)
            db.commit()
            return True
        return False

    @classmethod
    def exists(cls, db: Session, id: int) -> bool:
        """检查记录是否存在"""
        return db.query(cls).filter(cls.id == id).first() is not None

    @classmethod
    def get_all(cls,
                db: Session,
                skip: int = 0,
                limit: int = 100,
                include_deleted: bool = False) -> list["BaseModel"]:
        """获取所有记录"""
        query = db.query(cls)
        if not include_deleted:
            query = query.filter(cls.deleted_at.is_(None))
        return query.offset(skip).limit(limit).all()
