from ..models.base_model import BaseModel, CreateModel
from ..database import db


class QuestionBank(CreateModel):
    __tablename__ = 'question_banks'

    id = db.Column(db.Integer, primary_key=True)
    name = db.Column(db.String(100), nullable=False)
    description = db.Column(db.Text)
    subject = db.Column(db.String(50), nullable=False)
    creator_id = db.Column(db.Integer, db.ForeignKey('users.id'))
    create_time = db.Column(db.DateTime, default=db.func.now())
    update_time = db.Column(db.DateTime,
                            default=db.func.now(),
                            onupdate=db.func.now())
    is_deleted = db.Column(db.Boolean, default=False)
    is_public = db.Column(db.Boolean, default=False)  # 题库是否公开
    question_count = db.Column(db.Integer, default=0)  # 题目数量统计
    average_difficulty = db.Column(db.Float, default=0.0)  # 平均难度
    last_used_time = db.Column(db.DateTime)  # 最后使用时间

    # 关系定义
    creator = db.relationship('User', foreign_keys=[creator_id])
    questions = db.relationship('Question', backref='question_bank')
    permissions = db.relationship('QuestionBankPermission',
                                  backref='question_bank')  # 题库权限

    def __repr__(self):
        return f'<QuestionBank {self.id}: {self.name}>'

    def update_statistics(self):
        """更新题库统计信息"""
        self.question_count = len(self.questions)
        if self.question_count > 0:
            total_difficulty = sum(q.difficulty.level for q in self.questions)
            self.average_difficulty = total_difficulty / self.question_count
        else:
            self.average_difficulty = 0.0

    def has_permission(self, user_id: int, permission_type: str) -> bool:
        """检查用户是否有指定权限"""
        if self.creator_id == user_id:
            return True

        permission = next(
            (p for p in self.permissions
             if p.user_id == user_id and p.permission_type == permission_type),
            None)
        return permission is not None
