import jwt
from datetime import datetime, timedelta
from flask import current_app
from ..models.user import User
from ..models.auth_models import Token
from ..services.user_service import UserService
from ..store.auth_store import AuthStore


class AuthService:
    """认证服务"""

    @staticmethod
    def generate_token(user_id: int):
        """生成JWT Token并保存到数据库"""
        if not current_app.config.get('JWT_EXPIRATION_SECONDS'):
            raise ValueError('JWT_EXPIRATION_SECONDS config is required')
        if not current_app.config.get('SECRET_KEY'):
            raise ValueError('SECRET_KEY config is required')

        payload = {
            'user_id':
            user_id,
            'exp':
            datetime.utcnow() +
            timedelta(seconds=current_app.config['JWT_EXPIRATION_SECONDS'])
        }
        access_token = jwt.encode(payload,
                                  current_app.config['SECRET_KEY'],
                                  algorithm='HS256')
        refresh_payload = {
            'user_id': user_id,
            'exp': datetime.utcnow() + timedelta(days=7)
        }
        refresh_token = jwt.encode(refresh_payload,
                                   current_app.config['SECRET_KEY'],
                                   algorithm='HS256')

        # 创建并保存Token记录
        auth_store = AuthStore()
        token_data = {
            'access_token': access_token,
            'refresh_token': refresh_token,
            'expires_at': payload['exp'],
            'user_id': user_id
        }
        return auth_store.create_token(token_data)

    @staticmethod
    def login(username: str, password: str):
        """用户登录"""
        user = UserService.authenticate(username, password)
        if user:
            return AuthService.generate_token(user.id)
        return None

    @staticmethod
    def refresh_token(refresh_token: str):
        """刷新Token"""
        user_id = AuthService.verify_token(refresh_token)
        if user_id:
            return AuthService.generate_token(user_id)
        return None

    @staticmethod
    def get_current_user(token: str):
        """获取当前用户"""
        user_id = AuthService.verify_token(token)
        if user_id:
            auth_store = AuthStore()
            return auth_store.get_token_by_access_token(token).user
        return None
