from typing import List, Optional, Dict, Any
from datetime import datetime
from sqlalchemy.orm import Session, joinedload
from sqlalchemy import or_
from ..store.question_store import QuestionStore

# 基础模型
from ..models.question import Question
from ..models.question_type import QuestionType
from ..models.question_difficulty import QuestionDifficulty
from ..models.question_category import QuestionCategory
from ..models.question_option import QuestionOption
from ..models.user import User
from ..models.knowledge_point import KnowledgePoint

# 异常和常量
from ..common.exceptions import NotFoundException, PermissionDeniedException
from ..common.constants import QuestionStatus, ReviewStatus


class QuestionService:

    def __init__(self, db: Session):
        self.db = db
        self.question_store = QuestionStore(db)

    def create_question(self, creator_id: int,
                        question_data: Dict[str, Any]) -> Question:
        """创建题目"""
        question = self.question_store.create_question(
            creator_id=creator_id, question_data=question_data)
        return question

    def update_question(self, question_id: int, updater_id: int,
                        update_data: Dict[str, Any]) -> Question:
        """更新题目"""
        question = self.question_store.get_question_by_id(question_id)
        if not question:
            raise NotFoundException("题目不存在")

        # 检查更新权限
        if question.creator_id != updater_id:
            raise PermissionDeniedException("无权限更新该题目")

        question = self.question_store.update_question(question_id=question_id,
                                                       updater_id=updater_id,
                                                       update_data=update_data)
        return question

    def add_feedback(self, question_id: int, user_id: int, feedback_type: str,
                     content: str) -> Dict[str, Any]:
        """添加题目反馈"""
        result = self.question_store.add_feedback(question_id=question_id,
                                                  user_id=user_id,
                                                  feedback_type=feedback_type,
                                                  content=content)
        return result

    def get_question_statistics(self, question_id: int) -> Dict[str, Any]:
        """获取题目统计信息"""
        return self.question_store.get_question_statistics(question_id)

    def record_question_view(self, question_id: int, viewer_id: int) -> None:
        """记录题目查看"""
        self.question_store.record_question_view(question_id, viewer_id)

    def check_question_permission(self, question_id: int, user_id: int,
                                  required_permission: str) -> bool:
        """检查用户对题目的权限"""
        return self.question_store.check_question_permission(
            question_id, user_id, required_permission)

    def get_question_with_permission(self, question_id: int,
                                     user_id: int) -> Question:
        """获取题目详情（带权限检查）"""
        # 检查查看权限
        if not self.check_question_permission(question_id, user_id, "view"):
            raise PermissionDeniedException("无权限查看该题目")

        return self.get_question_detail(question_id)

    def update_question_with_permission(
            self, question_id: int, updater_id: int,
            update_data: Dict[str, Any]) -> Question:
        """更新题目（带权限检查）"""
        # 检查更新权限
        if not self.check_question_permission(question_id, updater_id,
                                              "update"):
            raise PermissionDeniedException("无权限更新该题目")

        return self.update_question(question_id, updater_id, update_data)

    def delete_question_with_permission(self, question_id: int,
                                        deleter_id: int) -> None:
        """删除题目（带权限检查）"""
        # 检查删除权限
        if not self.check_question_permission(question_id, deleter_id,
                                              "delete"):
            raise PermissionDeniedException("无权限删除该题目")

        self.delete_question(question_id, deleter_id)

    def resolve_feedback(self, feedback_id: int, resolver_id: int,
                         resolution: str) -> QuestionFeedback:
        """处理题目反馈"""
        return self.question_store.resolve_feedback(feedback_id, resolver_id,
                                                    resolution)

    def add_comment(self, question_id: int, user_id: int,
                    content: str) -> Dict[str, Any]:
        """添加题目评论"""
        return self.question_store.add_comment(question_id=question_id,
                                               user_id=user_id,
                                               content=content)

    def add_attachment(self, question_id: int, user_id: int, file_path: str,
                       file_name: str, file_type: str) -> Dict[str, Any]:
        """添加题目附件"""
        return self.question_store.add_attachment(question_id=question_id,
                                                  user_id=user_id,
                                                  file_path=file_path,
                                                  file_name=file_name,
                                                  file_type=file_type)

    def get_question_versions(self, question_id: int) -> List[QuestionVersion]:
        """获取题目版本历史"""
        return self.question_store.get_question_versions(question_id)

    def rollback_question_version(self, version_id: int,
                                  updater_id: int) -> Question:
        """回滚题目到指定版本"""
        return self.question_store.rollback_question_version(
            version_id, updater_id)

    def delete_attachment(self, attachment_id: int, deleter_id: int) -> None:
        """删除题目附件"""
        self.question_store.delete_attachment(attachment_id, deleter_id)

    def delete_comment(self, comment_id: int, deleter_id: int) -> None:
        """删除题目评论"""
        self.question_store.delete_comment(comment_id, deleter_id)

    def collect_question(self, question_id: int,
                         user_id: int) -> Dict[str, Any]:
        """收藏题目"""
        return self.question_store.collect_question(question_id, user_id)

    def uncollect_question(self, question_id: int,
                           user_id: int) -> Dict[str, Any]:
        """取消收藏题目"""
        return self.question_store.uncollect_question(question_id, user_id)

    def delete_question(self, question_id: int, deleter_id: int) -> None:
        """删除题目"""
        self.question_store.delete_question(question_id, deleter_id)

    def search_questions(self,
                         search_params: Dict[str, Any],
                         page: int = 1,
                         page_size: int = 20) -> Dict[str, Any]:
        """搜索题目"""
        return self.question_store.search_questions(search_params, page,
                                                    page_size)

    def get_question_detail(self, question_id: int) -> Question:
        """获取题目详情"""
        return self.question_store.get_question_detail(question_id)

    def add_knowledge_point(self, question_id: int,
                            knowledge_point_id: int) -> Dict[str, Any]:
        """为题目添加知识点"""
        return self.question_store.add_knowledge_point(question_id,
                                                       knowledge_point_id)

    def remove_knowledge_point(self, question_id: int,
                               knowledge_point_id: int) -> Dict[str, Any]:
        """移除题目的知识点"""
        return self.question_store.remove_knowledge_point(
            question_id, knowledge_point_id)

    def get_question_knowledge_points(
            self, question_id: int) -> List[KnowledgePoint]:
        """获取题目的知识点列表"""
        return self.question_store.get_question_knowledge_points(question_id)

    def review_question(self,
                        question_id: int,
                        reviewer_id: int,
                        status: str,
                        comment: Optional[str] = None) -> Question:
        """审核题目"""
        return self.question_store.review_question(question_id, reviewer_id,
                                                   status, comment)
