from ..models.user import User
from ..models.role import Role
from ..models.user_role import UserRole
from werkzeug.security import generate_password_hash, check_password_hash
from datetime import datetime
import re
from ..store.user_store import UserStore


class UserService:
    """用户服务"""

    @staticmethod
    def validate_password_strength(password: str):
        """验证密码强度"""
        if len(password) < 8:
            raise ValueError('密码长度至少8位')
        if not re.search(r'[A-Z]', password):
            raise ValueError('密码必须包含大写字母')
        if not re.search(r'[a-z]', password):
            raise ValueError('密码必须包含小写字母')
        if not re.search(r'[0-9]', password):
            raise ValueError('密码必须包含数字')
        if not re.search(r'[^A-Za-z0-9]', password):
            raise ValueError('密码必须包含特殊字符')

    @staticmethod
    def create_user(username: str, password: str, role_names: list = None):
        """创建用户"""
        user_store = UserStore()
        if user_store.get_user_by_username(username):
            raise ValueError('用户名已存在')

        UserService.validate_password_strength(password)

        user_data = {
            'username': username,
            'password_hash': generate_password_hash(password),
            'created_at': datetime.now()
        }

        # 分配默认角色
        if role_names:
            user_data['roles'] = role_names

        return user_store.create_user(user_data)

    @staticmethod
    def authenticate(username: str, password: str):
        """用户认证"""
        user_store = UserStore()
        user = user_store.get_user_by_username(username)
        if user and check_password_hash(user.password_hash, password):
            user_store.update_user(user.id, {'last_login': datetime.now()})
            return user
        return None

    @staticmethod
    def get_user_roles(user_id: int):
        """获取用户角色"""
        user_store = UserStore()
        user = user_store.get_user_by_id(user_id)
        return user.roles if user else []

    @staticmethod
    def update_user_password(user_id: int, new_password: str):
        """更新用户密码"""
        UserService.validate_password_strength(new_password)
        user_store = UserStore()
        return user_store.update_user(
            user_id, {'password_hash': generate_password_hash(new_password)})

    @staticmethod
    def set_user_status(user_id: int, is_active: bool):
        """设置用户状态"""
        user_store = UserStore()
        return user_store.update_user(user_id, {'is_active': is_active})

    @staticmethod
    def initiate_password_reset(username: str):
        """发起密码重置"""
        user_store = UserStore()
        user = user_store.get_user_by_username(username)
        if not user:
            return None

        # 生成重置token（有效期1小时）
        reset_token = generate_password_hash(
            f'{user.id}{datetime.now().timestamp()}')[:32]
        user_store.update_user(
            user.id, {
                'reset_token': reset_token,
                'reset_token_expires': datetime.now() + timedelta(hours=1)
            })
        return reset_token

    @staticmethod
    def reset_password(reset_token: str, new_password: str):
        """重置密码"""
        user_store = UserStore()
        user = user_store.get_user_by_reset_token(reset_token)
        if not user or user.reset_token_expires < datetime.now():
            return False

        UserService.validate_password_strength(new_password)
        return user_store.update_user(
            user.id, {
                'password_hash': generate_password_hash(new_password),
                'reset_token': None,
                'reset_token_expires': None
            })
