from datetime import datetime, timedelta
from typing import Union, Optional
import pytz


class DateUtil:
    """日期时间工具类"""

    @staticmethod
    def format_date(date: Union[datetime, str],
                    fmt: str = '%Y-%m-%d %H:%M:%S',
                    timezone: Optional[str] = None) -> str:
        """格式化日期时间
        
        Args:
            date: 日期时间对象或字符串
            fmt: 格式化字符串
            timezone: 时区名称
            
        Returns:
            格式化后的日期时间字符串
        """
        if isinstance(date, str):
            date = datetime.strptime(date, fmt)

        if timezone:
            tz = pytz.timezone(timezone)
            date = date.astimezone(tz)

        return date.strftime(fmt)

    @staticmethod
    def add_days(date: Union[datetime, str],
                 days: int,
                 fmt: str = '%Y-%m-%d %H:%M:%S') -> str:
        """日期加减天数
        
        Args:
            date: 日期时间对象或字符串
            days: 要加减的天数
            fmt: 格式化字符串
            
        Returns:
            计算后的日期时间字符串
        """
        if isinstance(date, str):
            date = datetime.strptime(date, fmt)

        new_date = date + timedelta(days=days)
        return new_date.strftime(fmt)

    @staticmethod
    def compare_dates(date1: Union[datetime, str],
                      date2: Union[datetime, str],
                      fmt: str = '%Y-%m-%d %H:%M:%S') -> int:
        """比较两个日期
        
        Args:
            date1: 第一个日期
            date2: 第二个日期
            fmt: 格式化字符串
            
        Returns:
            -1: date1 < date2
            0: date1 == date2
            1: date1 > date2
        """
        if isinstance(date1, str):
            date1 = datetime.strptime(date1, fmt)
        if isinstance(date2, str):
            date2 = datetime.strptime(date2, fmt)

        if date1 < date2:
            return -1
        elif date1 > date2:
            return 1
        return 0

    @staticmethod
    def get_current_time(timezone: Optional[str] = None) -> str:
        """获取当前时间
        
        Args:
            timezone: 时区名称
            
        Returns:
            当前时间字符串
        """
        now = datetime.now()
        if timezone:
            tz = pytz.timezone(timezone)
            now = now.astimezone(tz)
        return now.strftime('%Y-%m-%d %H:%M:%S')
